'' =================================================================================================
''
''   File....... jm_softrtc.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 04 JUL 2009
''   Updated.... 20 SEP 2009
''
'' =================================================================================================


con

  #0, RTC_MS, RTC_SECS, RTC_MINS, RTC_HRS, RTC_DAY  
  

var

  long  cog

  long  millis                                                  ' rtc registers
  long  seconds
  long  minutes
  long  hours
  long  day 


pub init : okay

'' Starts soft rtc -- time at midnight, Sunday

  finalize                                                      ' stop if running

  MS_001   := clkfreq / 1_000                                   ' setup cog vars   
  mspntr   := @millis   
  secspntr := @seconds 
  minspntr := @minutes 
  hrspntr  := @hours
  daypntr  := @day

  reset                                                         ' clear RTC

  okay := cog := cognew(@rtc, 0) + 1                            ' start cog


pub initx(s, m, h, d) : okay

'' Starts soft rtc with user-specified time and day

  finalize

  MS_001   := clkfreq / 1_000                                   ' setup cog vars   
  mspntr   := @millis   
  secspntr := @seconds 
  minspntr := @minutes 
  hrspntr  := @hours
  daypntr  := @day

  millis := 0                                                   ' clear ms
  set(s, m, h, d)                                               ' preset RTC
  
  okay := cog := cognew(@rtc, 0) + 1                            ' start cog     


pub finalize

'' Stops soft rtc; frees a cog 

  if cog
    cogstop(cog~ - 1)


pub set(s, m, h, d)

  seconds := s <# 59                                            ' validate/set RTC regs
  minutes := m <# 59
  hours   := h <# 23
  day     := d <# 6


pub reset

  longfill(@millis, 0, 5)                                       ' clear rtc regs
  

pub getms

  result := millis
  
  
pub getsecs

  result := seconds


pub getmins

  result := minutes


pub gethrs

  result := hours


pub getday

  result := day


pub todmins

'' Returns time of day in minutes
'' -- 0 to 1439 (00:00 to 23:59)

  result := (hours * 60) + minutes


pub rdreg(reg)

'' Read RTC register

  case reg
    RTC_MS   : result := millis
    RTC_SECS : result := seconds
    RTC_MINS : result := minutes
    RTC_HRS  : result := hours
    RTC_DAY  : result := day
    other    : result := -1


pub wrreg(reg, val)

'' Write RTC register

  case reg
    RTC_MS   : result := millis  := val <# 1_000
    RTC_SECS : result := seconds := val <# 59  
    RTC_MINS : result := minutes := val <# 59 
    RTC_HRS  : result := hours   := val <# 23 
    RTC_DAY  : result := day     := val <# 6
    other    : result := -1


dat

                        org     0

rtc                     mov     mstimer, MS_001                 ' load with 1ms
                        add     mstimer, cnt                    ' sync with system timer

waitms                  waitcnt mstimer, MS_001

updatems                rdlong  tmp1, mspntr                    ' get current milliseconds
                        add     tmp1, #1                        ' increment
                        cmp     tmp1, ONE_SEC           wc      ' 1000? (one second)
                if_ae   mov     tmp1, #0                        ' if yes, reset
                        wrlong  tmp1, mspntr                    ' update hub with new ms
                if_b    jmp     #waitms                         ' abort if no roll-over

updatesecs              rdlong  tmp1, secspntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wc
                if_ae   mov     tmp1, #0
                        wrlong  tmp1, secspntr
                if_b    jmp     #waitms

updatemins              rdlong  tmp1, minspntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wc
                if_ae   mov     tmp1, #0
                        wrlong  tmp1, minspntr
                if_b    jmp     #waitms

updatehrs               rdlong  tmp1, hrspntr
                        add     tmp1, #1
                        cmp     tmp1, #24               wc
                if_ae   mov     tmp1, #0
                        wrlong  tmp1, hrspntr
                if_b    jmp     #waitms

updateday               rdlong  tmp1, daypntr
                        add     tmp1, #1
                        cmp     tmp1, #7                wc 
                if_ae   mov     tmp1, #0                 
                        wrlong  tmp1, daypntr
                        jmp     #waitms 

' --------------------------------------------------------------------------------------------------

ONE_SEC                 long    1_000 

MS_001                  long    0-0

mspntr                  long    0-0  
secspntr                long    0-0  
minspntr                long    0-0  
hrspntr                 long    0-0  
daypntr                 long    0-0  

mstimer                 res     1
tmp1                    res     1   

                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   